#ifndef NEARDETDAQCHANNELMAP_H
#define NEARDETDAQCHANNELMAP_H
#include "DAQChannelMap/DAQChannelMap.h"

namespace daqchannelmap
{
  class NearDetDAQChannelMap: public DAQChannelMap
  {
    // NearDet specific functions and mapping. 
  public:
    dchan encodeDChan(lchan logicalchan) const;
    lchan encodeLChan(dchan daqchan) const;

    block_t      computeBlock(lchan logicalchan)        const;
    plane_t      computePlaneInBlock(lchan logicalchan) const;
    unsigned int computeModule(lchan logicalchan)       const;
    fiber_t      computeFiberInModule(dchan daqchan)    const;
    
    DetView_TYPE getView(dchan daqchan)           const;
    unsigned int getTotalNumberOfDiblocks()       const;
    unsigned int getNumberOfDCM(DiBlock_TYPE dbt) const;
    DiBlock_TYPE getDiBlockType(dchan chan)       const;
    
    NearDetDAQChannelMap();
    virtual ~NearDetDAQChannelMap();

  private:
    dcm_id_t computeDCM(plane_t   plane,
			fiber_t   fiber,
			diblock_t diblock) const;
    feb_t    computeFEB(plane_t   plane,
			fiber_t   fiber,
			diblock_t diblock) const;
    pixel_t  computePixel(fiber_t   fiber) const;
    plane_t  computePlane(diblock_t diblock,
			  dcm_id_t  dcm,
			  feb_t     feb) const;
    cell_t   computeCell(diblock_t diblock,
			 dcm_id_t  dcm,
			 feb_t     feb,
			 pixel_t   pixel) const;
    pixel_t  FiberToPixel(fiber_t fiber) const;
    fiber_t  PixelToFiber(pixel_t pixel) const;
    
    bool printBinary(uint32_t) const;
    bool printHex(uint32_t)    const;
  };
}
#endif
////////////////////////////////////////////////////////////////////////
