#ifndef RAWCONFIGURATIONNAME_H
#define RAWCONFIGURATIONNAME_H
#include "DAQDataFormats/RawConfigurationNameV0.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationName
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfname
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace rawconfname {
        const uint32_t MAXIMUM_SIZE = 1000;
        static const version_t RAWCONFIGURATIONNAME_LATEST_VERSION = LAST_CLASS_VERSION;
    }

    class RawConfigurationName: public PREVIOUS_VERSION_CLASS_NAME{
        public:

            /// Constructors
            RawConfigurationName (const version_t version = rawconfname::RAWCONFIGURATIONNAME_LATEST_VERSION);                             //! The default constructor
            RawConfigurationName (const RawConfigurationName& copy_in);  //! Deep copy constructor
            RawConfigurationName (const void* dadablockheader_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION); //! Shallow copy constructor
            /// Destructor
            ~RawConfigurationName();

            IMPLEMENT_MAIN_STANDARD
            IMPLEMENT_MAIN_setBufferSource

            IMPLEMENT_GET_MAIN(Version   , uint32_t)
            IMPLEMENT_GET_MAIN(Size      , uint32_t)
            IMPLEMENT_GET_MAIN(HeaderSize, uint32_t)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(getName, const, uint32_t, std::string&)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setName,      , uint32_t, const std::string&)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(getChar1, const, uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_2PAR(setChar1,      , bool    , const uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(getChar2, const, uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_2PAR(setChar2,      , bool    , const uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(getChar3, const, uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_2PAR(setChar3,      , bool    , const uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(getChar4, const, uint32_t, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_2PAR(setChar4,      , bool    , const uint32_t, const uint32_t)

        protected:
            IMPLEMENT_SET_MAIN(Version   , uint32_t)
            IMPLEMENT_SET_MAIN(Size      , uint32_t)
            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawConfigurationName)
            bool setFunctionPointers();

    };

} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONNAME_H */
