#ifndef RAWMILLISLICE_H
#define RAWMILLISLICE_H

#include "DAQDataFormats/RawMilliSliceHeader.h"
#include "DAQDataFormats/RawMilliSliceIndex.h"
#include "DAQDataFormats/RawMicroSlice.h"


namespace daqdataformats {

  static const uint32_t DEFAULT_MILLISLICE_SIZE = 262144; /* Default allocated MilliSlice Buffer Size */


  class RawMilliSlice : public RawDAQData {
    
        public:
            // Constructors
            RawMilliSlice (const uint32_t bufsize=DEFAULT_MILLISLICE_SIZE, const uint32_t tlvl=0); //! The default constructor
            RawMilliSlice (const RawMilliSlice& copy_in);  //! Deep copy constructor
            RawMilliSlice (const void* millislice_buffer, const uint32_t buffersize = DEFAULT_MILLISLICE_SIZE); //! Shallow copy constructor
            /// Destructor
            ~RawMilliSlice();

            void     init            ();
            uint32_t sizeofdata      () const {return _Header->getSliceSize();}
            bool     setBufferSource(const void*);

            using RawDAQData::readData;	// unhide the reset of the readData methods
            readDataStatus    readData(const int fd);

            /// Add some data containing NumberofMicroslices MicroSlices into the MilliSlice changing the MilliSlice Header appropriately
            /// if indexate is true, produce an index for the MicroSlice
            /// If check_mc_flag = true, the method checks whether the microslice is MC generated
            /// and adds an appropriate flag to the MilliSlice Header.
            /// bool check_mc_flag = true
            bool addMicroSlice(const void* microslice_buffer, const bool indexate=false, const bool check_mc_flag=false);
            /// Add some data containing NumberofMicroslices MicroSlices into the MilliSlice changing the MilliSlice Header appropriately
            /// If check_mc_flag = true, the method checks whether the microslice is MC generated
            /// and adds an appropriate flag to the MilliSlice Header.
            /// bool check_mc_flag = true
            bool addData(const void* buffer, const uint32_t size, const uint32_t number_of_microslices, const bool check_mc_flag=false);


            bool         clear();     //!< Clear Internal Buffer and zero out _fd_read_bytes
            bool         softclear(); //!< zero out _fd_read_bytes, but leaving buffer alone

            daqdataformats::RawMilliSliceHeader* getHeader()             const {return _Header;}
            daqdataformats::RawMicroSlice*       getFloatingMicroSlice() const {return _FloatingMicroSlice;}
            daqdataformats::RawMilliSliceIndex*  getMilliSliceIndex() const;

            uint32_t getVersion() 			const {return _Header->getVersion();}
            uint32_t getMarker() 			const {return _Header->getMarker();}
            uint32_t getDestBuffID() 			const {return _Header->getDestBuffID();}
            uint32_t getPartition() 			const {return _Header->getPartition();}
            uint64_t getTimeStart() 			const {return _Header->getTimeStart();}
            uint32_t getTimeStartLow() 			const {return _Header->getTimeStartLow();}
            uint32_t getTimeStartHigh() 		const {return _Header->getTimeStartHigh();}
            uint32_t getTimeLength() 			const {return _Header->getTimeLength();}
            rawmillihdr::Mode_t getMode() 		const {return _Header->getMode();}
            uint32_t getMilliSliceSEQ() 		const {return _Header->getMilliSliceSEQ();}
            uint32_t getFrameSEQ() 			const {return _Header->getFrameSEQ();}
            uint32_t getSliceSize() 			const {return _Header->getSliceSize();}
            uint32_t getMicrosliceNumber() 		const {return _Header->getMicrosliceNumber();}
            uint32_t getDiblockId() 			const {return _Header->getDiblockId();}
            uint32_t getDCMId() 			const {return _Header->getDCMId();}
            uint32_t getDetId() 			const {return _Header->getDetId();}
	    uint32_t getGlobalDCMID() 			const {return _Header->getGlobalDCMID();}
            uint32_t getMilliSliceIndexNumber() 	const {return _Header->getMilliSliceIndexNumber();}
            uint32_t getMicroSliceTimeLength_usec() 	const {return _Header->getMicroSliceTimeLength_usec();}
            bool     getMonteCarloFlag() 		const {return _Header->getMonteCarloFlag();}
            uint32_t isComplete() 			const {return _Header->isComplete();}
            uint32_t isCRCCalculationUsed() 		const {return _Header->isCRCCalculationUsed();}
            uint32_t isMilliSliceIndexUsed() 		const {return _Header->isMilliSliceIndexUsed();}
            uint32_t isConnectionInitialization() 	const {return _Header->isConnectionInitialization();}
            uint32_t isCloseConnection() 		const {return _Header->isCloseConnection();}
            uint32_t isCorruptData()                    const {return _Header->isCorruptData();}
            uint32_t isMissingData()                    const {return _Header->isMissingData();}
            uint32_t getCRC() const;

            bool     setDiblockId(const uint32_t a)			{return _Header->setDiblockId(a);}
            bool     setDCMId(const uint32_t a)				{return _Header->setDCMId(a);}
            bool     setDetId(const uint32_t a)				{return _Header->setDetId(a);}
            bool     setGlobalDCMID(const uint32_t a)			{return _Header->setGlobalDCMID(a);}
            bool     setDestBuffID(const uint32_t a)			{return _Header->setDestBuffID(a);}
            bool     setPartition(const uint32_t a)			{return _Header->setPartition(a);}
            bool     setTimeStart(const uint64_t a)			{return _Header->setTimeStart(a);}
            bool     setTimeStartLow(const uint32_t a)			{return _Header->setTimeStartLow(a);}
            bool     setTimeStartHigh(const uint32_t a)			{return _Header->setTimeStartHigh(a);}
            bool     setTimeLength(const uint32_t a)			{return _Header->setTimeLength(a);}
            bool     setComplete(const bool a)				{return _Header->setComplete(a);}
            bool     setMode(const rawmillihdr::Mode_t a)		{return _Header->setMode(a);}
            bool     setConnectionInitialization(const bool a)		{return _Header->setConnectionInitialization(a);}
            bool     setCloseConnection(const bool a)			{return _Header->setCloseConnection(a);}
            bool     setCorruptData(const bool a)			{return _Header->setCorruptData(a);}
            bool     setMissingData(const bool a)			{return _Header->setMissingData(a);}
            bool     setMilliSliceSEQ(const uint32_t a)			{return _Header->setMilliSliceSEQ(a);}
            bool     setMicroSliceTimeLength_usec(const uint32_t a)	{return _Header->setMicroSliceTimeLength_usec(a);}
            bool     setMonteCarloFlag(const bool a)			{return _Header->setMonteCarloFlag(a);}
            bool     setMilliSliceIndexUsed(const bool a)		{return _Header->setMilliSliceIndexUsed(a);}
            bool     setFrameSEQ(const uint32_t a)			{return _Header->setFrameSEQ(a);}

            bool     setFloatingMicroSlice(const uint32_t) const;
            bool     close(const bool produce_crc=false);

        protected:

            version_t figureOutTheVersion(const void*) const {return 0;}
            bool      setFunctionPointers()                  {return true;}
            uint32_t  predefinedSize() const                 {return rawmillihdr::MILLISLICEHEADER_SIZE;}
            bool      setFloatingMicroSliceToBufferSource(const void* ) const;

        protected:

            // These have to have a different name from readData and print.
            // Otherwise, we can't bind them to their respactive DELEGATEs.
            void*    readDataFromBuffer(const void*);
            void     printStandard     (std::ostream& os=std::cout) const;

            // The actual internal buffer (not a pointer).
            // Used only if the constructors with the default size are used

            daqdataformats::RawMilliSliceHeader*           _Header;                  //! MilliSlice Header
            mutable daqdataformats::RawMilliSliceIndex*    _Index;                   //! MilliSlice Index

            mutable daqdataformats::RawMicroSlice*         _FloatingMicroSlice;      //! Floating MicroSlice
            mutable int                                    _FloatingMicroSliceNumber;//! Floating MicroSlice Number


            size_t                  _fd_read_bytes;

            uint32_t                _tlvl; // debug "trace level mask"
            void tprintf( unsigned lvl, const char *fmt, ... )
                __attribute__((format(printf, 3, 4))); // g++ note: attributes are not

            
  };

  
} // END NAMESPCE daqdataformats
#endif /* RAWMILLISLICE_H */
