#ifndef RAWMILLISLICEHEADER_H
#define RAWMILLISLICEHEADER_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/Macros.h"

#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMilliSliceHeader
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmillihdr

namespace daqdataformats {


    namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t MILLISLICEHEADER_SIZE          = 9;
            static const uint32_t MILLISLICEHEADER_SIZE_PLUS_ONE = MILLISLICEHEADER_SIZE + 1;

            enum Mode_t{
                Mode_Keep_All_MicroSlices      = 0,
                Mode_Discard_Empty_MicroSlices = 1,
            };

            // MilliSlice header delimiter. [0] value in the header, [1] value in the tail
            static const uint32_t Marker_value[2] = {0x00007777, 0x00007778};

            
            enum MilliSliceHeaderMASKS{
                MASK_Marker                   = 0xFFFF0000, ///< Marker Mask                 (16 bits)
                MASK_Version                  = 0x00000FFF, ///< Version Mask                (12 bits)
                MASK_SliceSize                = 0xFFFFFFFF, ///< MilliSlice Size Mask       (32 bits)
                MASK_Partition                = 0xFC000000, ///< Partition                   ( 6 bits)
                MASK_DestBuffID               = 0x03FF0000, ///< Dest Buffer ID Mask         (10 bits)
                MASK_DiblockId                = 0x000000FF, // Diblock ID
                MASK_DCMId                    = 0x00001F00, // DCM ID
                MASK_DetId                    = 0x0000E000, // Detector ID
                MASK_GlobalDCMID              = 0x0000FFFF,  // Global ID
                MASK_MilliSliceSEQ            = 0xFFFFFFFF, ///< Slice Seq Mask              (32 bits)
                MASK_TimeStartLow             = 0xFFFFFFFF, ///< Start Time Low Mask         (32 bits)
                MASK_TimeStartHigh            = 0x00FFFFFF, ///< Start Time High Mask        (24 bits)
                MASK_TimeLength               = 0x03FFFFFF, ///< MilliSlice Time Length Mask (26 bits)
                MASK_Complete                 = 0x04000000, ///< Completetess bit Mask
                MASK_Mode                     = 0x08000000, ///< DCM Mode Mask
                MASK_ConnectionInitialization = 0x10000000, ///< Initialization Connection bit Mask
                MASK_CloseConnection          = 0x20000000, ///< Close Connection bit
                MASK_CorruptData              = 0x40000000, ///< Corrupt data bit
                MASK_MissingData              = 0x80000000, ///< Missing data bit
                MASK_MicrosliceNumber         = 0x003FFFFF, ///< Number of MicroSlices Mask  (22 bits)
                MASK_FrameSEQ                 = 0xFFC00000, ///< Frame Seq Mask              (10 bits)
                MASK_MilliSliceIndexNumber    = 0x0000FFFF, ///< The number of MilliSlice index entries (16 bits)
                MASK_MicroSliceTimeLength_usec= 0x0FFF0000, ///< The conf'd time length of MicroSlices, in usec. (12 bits)
                MASK_MonteCarloFlag           = 0x20000000, ///< SIM flag                    (1 bit)
                MASK_CRCCalculationUsed       = 0x40000000, ///< Is CRC calculation used?    (1 bit)
                MASK_MilliSliceIndexUsed      = 0x80000000, ///< Do we EVER produce the MilliSlice index? (1 bit)
            };


            enum MilliSliceHeaderWORDS{
                WORD_Marker                   = 0,
                WORD_Version                  = 0,
                WORD_SliceSize                = 1,
                WORD_Partition                = 2,
                WORD_DestBuffID               = 2,
                WORD_DiblockId                = 2, // Diblock ID
                WORD_DCMId                    = 2, // DCM ID
                WORD_DetId                    = 2, // Detector ID
                WORD_GlobalDCMID              = 2, // Global ID
                WORD_MilliSliceSEQ            = 3,
                WORD_TimeStartLow             = 4,
                WORD_TimeStartHigh            = 5,
                WORD_TimeLength               = 6,
                WORD_Complete                 = 6,
                WORD_Mode                     = 6,
                WORD_ConnectionInitialization = 6,
                WORD_CloseConnection          = 6,
                WORD_CorruptData              = 6,
                WORD_MissingData              = 6,
                WORD_MicrosliceNumber         = 7,
                WORD_FrameSEQ                 = 7,
                WORD_MilliSliceIndexNumber    = 8,
                WORD_MicroSliceTimeLength_usec= 8,
                WORD_MonteCarloFlag           = 8,
                WORD_CRCCalculationUsed       = 8,
                WORD_MilliSliceIndexUsed      = 8,
            };

            DEFINE_SHIFT_FROM_MASK(Marker)
            DEFINE_SHIFT_FROM_MASK(Version)
            DEFINE_SHIFT_FROM_MASK(SliceSize)
            DEFINE_SHIFT_FROM_MASK(Partition)
            DEFINE_SHIFT_FROM_MASK(DestBuffID)
            DEFINE_SHIFT_FROM_MASK(DiblockId)
            DEFINE_SHIFT_FROM_MASK(DCMId)
            DEFINE_SHIFT_FROM_MASK(DetId)
            DEFINE_SHIFT_FROM_MASK(GlobalDCMID)
            DEFINE_SHIFT_FROM_MASK(MilliSliceSEQ)
            DEFINE_SHIFT_FROM_MASK(TimeStartLow)
            DEFINE_SHIFT_FROM_MASK(TimeStartHigh)
            DEFINE_SHIFT_FROM_MASK(TimeLength)
            DEFINE_SHIFT_FROM_MASK(Complete)
            DEFINE_SHIFT_FROM_MASK(Mode)
            DEFINE_SHIFT_FROM_MASK(ConnectionInitialization)
            DEFINE_SHIFT_FROM_MASK(CloseConnection)
            DEFINE_SHIFT_FROM_MASK(CorruptData)
            DEFINE_SHIFT_FROM_MASK(MissingData)
            DEFINE_SHIFT_FROM_MASK(MicrosliceNumber)
            DEFINE_SHIFT_FROM_MASK(FrameSEQ)
            DEFINE_SHIFT_FROM_MASK(MilliSliceIndexNumber)
            DEFINE_SHIFT_FROM_MASK(MicroSliceTimeLength_usec)
            DEFINE_SHIFT_FROM_MASK(MonteCarloFlag)
            DEFINE_SHIFT_FROM_MASK(CRCCalculationUsed)
            DEFINE_SHIFT_FROM_MASK(MilliSliceIndexUsed)

    }// end of namespace rawmillihdr


    
  class RawMilliSliceHeader: public RawDAQData {
    
        public:
    
            // Constructors
            RawMilliSliceHeader(); //! The default constructor
            RawMilliSliceHeader(const RawMilliSliceHeader& copy_in);  //! Deep copy constructor
            RawMilliSliceHeader(const void* millisliceheader_buffer); //! Shallow copy constructor
            /// Destructor
            ~RawMilliSliceHeader();

            void     init            ();
            uint32_t sizeofdata      () const { return rawmillihdr::MILLISLICEHEADER_SIZE; }

            uint64_t            getTimeStart() const;
            rawmillihdr::Mode_t getMode() const;
            bool                setTimeStart(const uint64_t);
            bool                setMode(const rawmillihdr::Mode_t);
            bool                addSliceSize(const uint32_t);

            bool                advanceMicrosliceNumber();
            bool                advanceFrameSEQ();
            bool                resetFrameSEQ();
            bool                advanceMilliSliceSEQ();

            IMPLEMENT_GET_NOVERSION          (Version, uint32_t)
            IMPLEMENT_SET_NOVERSION_PROTECTED(Version, uint32_t)

            IMPLEMENT_GET_AND_SET_NOVERSION(Marker, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(DestBuffID, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(Partition, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(DiblockId, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(DCMId, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(DetId, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(GlobalDCMID, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(TimeStartLow, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(TimeStartHigh, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(TimeLength, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(MilliSliceSEQ, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(FrameSEQ, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(SliceSize, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(MicrosliceNumber, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(MilliSliceIndexNumber, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(MicroSliceTimeLength_usec, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(MonteCarloFlag, bool)

            IMPLEMENT_IS_AND_SET_NOVERSION(Complete)
            IMPLEMENT_IS_AND_SET_NOVERSION(ConnectionInitialization)
            IMPLEMENT_IS_AND_SET_NOVERSION(CloseConnection)
            IMPLEMENT_IS_AND_SET_NOVERSION(CorruptData)
            IMPLEMENT_IS_AND_SET_NOVERSION(MissingData)
            IMPLEMENT_IS_AND_SET_NOVERSION(CRCCalculationUsed)
            IMPLEMENT_IS_AND_SET_NOVERSION(MilliSliceIndexUsed)

            using     RawDAQData::readData;// unhide the reset of the readData methods
            void*     readData(const void* buffer){return readDataGeneral(buffer);}

        private:

            // These have to have a different name from readData and print.
            // Otherwise, we can't bind them to their respactive DELEGATEs.
            void*     readDataFromBuffer(const void* buffer){return readDataGeneral(buffer);}
            void      printStandard     (std::ostream& os=std::cout) const;

            version_t figureOutTheVersion(const void*) const {return 0;}
            bool      setFunctionPointers()                  {return true;}
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWMILLISLICEHEADER_H */
