#ifndef RAWSUMMARYEVENTTAILV0_H
#define RAWSUMMARYEVENTTAILV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryEventTail
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawsummaryeventtail
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {

        // MilliSlice header delimiter. [0] value in the header, [1] value in the tail
        static const uint32_t Marker_value[3] = {0xFFEE, 0x929F, 0x929F};
    }

namespace  VERSION_NAMESPACE {
   namespace CURRENT_FIELDS_SUB_NAMESPACE {


            static const uint32_t RAWSUMMARYEVENTTAIL_SIZE = 4;

            enum SummaryEventTailMASKS{
                MASK_EventNumberHi = 0x000000FF,//! Universal event number higher 32-39 bits
                MASK_Version       = 0x0000FF00,//! Version
                MASK_Marker        = 0xFFFF0000,// Marker Mask                 (16 bits)
                MASK_EventNumberLo = 0xFFFFFFFF,//! Universal event number lower  0-31 bits
                MASK_EventSize     = 0xFFFFFFFF, //! Size of the event in words  (8 bits)
                MASK_MarkerRight   = 0x0000FFFF,// Marker Mask                 (16 bits)
                MASK_MarkerLeft    = 0xFFFF0000// Marker Mask                 (16 bits)
            };

            enum SummaryEventTailWORDS{
                WORD_EventNumberHi = 0,
                WORD_Version       = 0,
                WORD_Marker        = 0,
                WORD_EventNumberLo = 1,
                WORD_EventSize     = 2,
                WORD_MarkerRight   = 3,
                WORD_MarkerLeft    = 3
            };

            DEFINE_SHIFT_FROM_MASK(EventNumberHi)
            DEFINE_SHIFT_FROM_MASK(Version)
            DEFINE_SHIFT_FROM_MASK(Marker)
            DEFINE_SHIFT_FROM_MASK(EventNumberLo)
            DEFINE_SHIFT_FROM_MASK(EventSize)
            DEFINE_SHIFT_FROM_MASK(MarkerRight)
            DEFINE_SHIFT_FROM_MASK(MarkerLeft)
        
  } 
  
    class RawSummaryEventTail: public PREVIOUS_VERSION_CLASS_NAME {
        public:
            RawSummaryEventTail (const version_t); ///< The default constructor
            RawSummaryEventTail (const RawSummaryEventTail&);
            ~RawSummaryEventTail(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const { return CURRENT_FIELDS_SUB_NAMESPACE::RAWSUMMARYEVENTTAIL_SIZE;}
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

        protected:
            IMPLEMENT_FIRST_GET_AND_SET(Marker       , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(MarkerLeft   , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(MarkerRight  , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Version      , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(EventNumberLo, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(EventNumberHi, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(EventSize    , uint32_t)

  };
  
  
}} // END NAMESPACE daqdataformats
#endif /* RAWSUMMARYEVENTTAILV0_H */
