#ifndef DAQRAWTRIGGERRANGEV0_H
#define DAQRAWTRIGGERRANGEV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/TriggerDefines.h"
#include "DAQDataFormats/Macros.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerRange
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggerrange
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

namespace  VERSION_NAMESPACE {
    namespace CURRENT_FIELDS_SUB_NAMESPACE {

        static const uint32_t TRIGGERRANGE_SIZE = 1;

        enum TriggerRangeMASKS{
            MASK_TriggerLength = 0x0FFFFFFF,
            MASK_Version       = 0xF0000000
        };
        enum TriggerRangeWORDS{
            WORD_TriggerLength = 0,
            WORD_Version       = 0
        };

        DEFINE_SHIFT_FROM_MASK(TriggerLength)
        DEFINE_SHIFT_FROM_MASK(Version)
    }// end of namespace triggerrange

  //! Class to hold the data from the global trigger range in the correct format
  /*!
    The RawTriggerRangeV0 contains the width of the trigger gate in
    500 nsec units.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerRange: public PREVIOUS_VERSION_CLASS_NAME {

        public:

            RawTriggerRange (const version_t); ///< The default constructor
            RawTriggerRange (const RawTriggerRange&);
            ~RawTriggerRange(); ///< Destructor

            void     init      ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print     (std::ostream& os=std::cout) const;
            uint32_t sizeofdata() const { return CURRENT_FIELDS_SUB_NAMESPACE::TRIGGERRANGE_SIZE;}

            /// Nanoslice of version 0 is a fixed size format
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            IMPLEMENT_FIRST_GET_AND_SET(Version      , uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(TriggerLength, uint32_t);
  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGERRANGEV0_H */
