#ifndef TIME_STAMP_COUNTER
#define TIME_STAMP_COUNTER
///////////////////////////////////////////
// Define the High Res Time Stamp Counter
///////////////////////////////////////////

#include <stdint.h>

/*
 * High Resolution Time Stamp Counter 
 * Based on the intel cpu's clock 
 * register.  CPU must support the rtdsc
 * instruction
 */

#if ( __i386__ | __k8__ | __amd64__ ) /* SYSTEM ARCH */

typedef union {
  unsigned long long full_time;
  uint32_t time[2];
} cycle_t;

//typedef unsigned long long cycles_t; /* Definte a data type to hold the register value */

/* Define the assembler instruction */
#define rdtscll(val)				\
 __asm__ __volatile__ ("rdtsc" : "=A" (val))

/* The function tsc will now return the value of the register properly */
inline unsigned long long tsc(){
  unsigned long long ret;
  rdtscll(ret);
  return ret;
};

#else
#if ( __PPC__ | __powerpc__ )

typedef union {
  unsigned long long full_time;
  uint32_t time[2];
} cycle_t;

#warning "PowerPC-603e Clock Register Not Implimented -- NOP inserted"
#define rdtscll(val)				\
 __asm__ __volatile__ ("nop")

/* The function tsc will now return the value of the register properly */
inline unsigned long long tsc(){
  unsigned long long ret = 0;
  rdtscll(ret);
  return ret;
};
#endif /* PPC ARCH */
#endif /* SYSTEM ARCH */

float CPUSpeed();

#endif /* TIME_STAMP_COUNTER */
///////////////////////////////////////////
// END Def High Res Time Stamp Counter
///////////////////////////////////////////
