#include "DAQDataFormats/RawEvent.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEvent
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawevent
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats{

//------------------------------------------------------------------------------
// Default constructor
RawEvent::RawEvent(const uint32_t size, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    reserveInternalBuffer(size);
    defaultConstructorFloatingSize();
}
//------------------------------------------------------------------------------
RawEvent::RawEvent(const RawEvent& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawEvent::RawEvent(const void* buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    copy(buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawEvent::~RawEvent() { }

/// All binding routines
#define STANDARD_ROUTINES                           \
            BIND_CUSTOM_FUNCTION(nextEvent, version0, , bool, );			\
            BIND_CUSTOM_FUNCTION(setAppropriateMCFlag              , version0,      , bool, const void*);\
            BIND_CUSTOM_FUNCTION(addDataBlock                      , version0, 	    , bool, const void*);\
            BIND_CUSTOM_FUNCTION(addDataBlockWithSize              , version0, 	    , bool, const void*, const uint32_t);\
            BIND_CUSTOM_FUNCTION(close                             , version0, 	    , bool, const bool);\
            BIND_CUSTOM_FUNCTION(setTrigger1                       , version0, 	    , bool, const daqdataformats::RawTrigger&);\
            BIND_CUSTOM_FUNCTION(setTrigger2                       , version0, 	    , bool, const void*);\
            BIND_CUSTOM_FUNCTION(setFloatingDataBlockToBufferSource, version0, const, bool, const void*);\
            BIND_CUSTOM_FUNCTION(getHeader                         , version0, const, daqdataformats::RawEventHeader*, ); \
            BIND_CUSTOM_FUNCTION(getTail                           , version0,      , daqdataformats::RawEventTail*, ); \
            BIND_CUSTOM_FUNCTION(getTrigger                        , version0, 	    , daqdataformats::RawTrigger*, );   \
            BIND_CUSTOM_FUNCTION(getFloatingDataBlock              , version0, 	    , daqdataformats::RawDataBlock*, ); \
            BIND_CUSTOM_FUNCTION(getHeaderConst                    , version0, const, daqdataformats::RawEventHeader*, ); \
            BIND_CUSTOM_FUNCTION(getTailConst                      , version0, const, daqdataformats::RawEventTail*, ); \
            BIND_CUSTOM_FUNCTION(getTriggerConst                   , version0, const, daqdataformats::RawTrigger*, ); \
            BIND_CUSTOM_FUNCTION(getFloatingDataBlockConst         , version0, const, daqdataformats::RawDataBlock*, ); \
            BIND_CUSTOM_FUNCTION(getHeaderMarkerLeft               , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getHeaderMarkerRight              , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getVersion                        , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getDataLoggerID                   , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getRunNumber                      , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getRunType                        , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getConfigIDX                      , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getPartition                      , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getSubrun                         , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getEventNumberLo                  , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getEventNumberHi                  , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getEventNumber                    , version0, const, uint64_t, ); \
            BIND_CUSTOM_FUNCTION(getEventType                      , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getEventSize                      , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getDataBlockNumber                , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(isEventIncomplete                 , version0, const, bool, ); \
            BIND_CUSTOM_FUNCTION(getMonteCarloFlag                 , version0, const, bool, ); \
            BIND_CUSTOM_FUNCTION(isCRCCalculationUsed              , version0, const, bool, ); \
            BIND_CUSTOM_FUNCTION(setTriggerPointer                 , version0,      , bool, const void*);\
            BIND_CUSTOM_FUNCTION(setDataLoggerID                   , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setRunNumber                      , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setRunType                        , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setConfigIDX                      , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setPartition                      , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setSubrun                         , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setEventNumberLo                  , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setEventNumberHi                  , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setEventNumber                    , version0, 	    , bool, const uint64_t);\
            BIND_CUSTOM_FUNCTION(setEventType                      , version0, 	    , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setEventIncomplete                , version0, 	    , bool, const bool);\
            BIND_CUSTOM_FUNCTION(setMonteCarloFlag                 , version0, 	    , bool, const bool);\
            BIND_CUSTOM_FUNCTION(setFloatingDataBlock              , version0, const, bool, const uint32_t);\
            BIND_STADARD_FUNCTIONS;\
            _tailSize = getTailSize(_version);

#include "DAQDataFormats/setFunctionPointers_template.inc"


}
