#include "DAQDataFormats/RawEventV1.h"
#include "DAQDataFormats/RawEvent.h"
#include <boost/format.hpp>
/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEvent
#define  CURRENT_CLASS_VERSION        1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawevent
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------------------------------------------------------------
// Default constructor
RawEvent::RawEvent(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{ }

//------------------------------------------------------------------------------
RawEvent::RawEvent(const RawEvent& copyin):
    PREVIOUS_VERSION_CLASS_NAME(copyin)
{ }

//------------------------------------------------------------------------------
// Destructor
RawEvent::~RawEvent() { }


//------------------------------------------------------------------------------
// Initialization of Data members
void RawEvent::init(){
    version0::RawEvent::init();
    
    /// Tail should always be non-filled at initialization
    _Header->setTailFilled(0);

    if(_Trigger){
         _Header->setTriggerFilled(1);
    }
    else{
         _Header->setTriggerFilled(0);
    }
}


//------------------------------------------------------------------------------
bool RawEvent::nextEvent(){
    
    _Header->setTailFilled(0);

    return version0::RawEvent::nextEvent();
}

//------------------------------------------------------------------------------
// Usually just packs the Header leaving the rest of the _Buffer unchanged.
// Also, puts the Header in the tail as well. Puts CRC in the very end.
bool RawEvent::close(const bool produce_crc){
    _Header->setTailFilled(1);
    return version0::RawEvent::close(produce_crc);
}
//------------------------------------------------------------------------------
// Setters
// Different setters for the trigger
bool RawEvent::setTrigger2       (const void*  pointer) {
        
        version0::RawEvent::setTrigger2(pointer);

        return _Header->setTriggerFilled(1);
}

//------------------------------------------------------------------------------
bool RawEvent::setBufferSource(const void* pointer){

        _Buffer  = (void*)pointer;

        _shouldBufferBeInternal = isInternalBuffer();

        /// If Header is already defined, then set its buffer source
        /// Otherwise, need to construct it
        if(_Header){
            _Header->setBufferSource(pointer);

            EXECUTE_FUNCTION(setTriggerPointer)(&RAWBUFF32[_Header->sizeofdata()]);
        }/// end of if there is Header
        else{
        /// So, the header is not constructed. Construct it now

            PRINT_ON_DEBUG("Setting Header "<<"version = "<<_version)
            _Header = new daqdataformats::RawEventHeader(_Buffer, _version); ///< version of the EventHeader equals the version of the Event

            /// If the buffer is internal, need to resize it just in case,
            /// because TriggerFilled and TailFilled must be 0 by default
            if(_shouldBufferBeInternal){
                resizeInternalBuffer(_Header->sizeofdata() + 1);
                /// Check pointers after each read or resize buffer
                checkBufferInternalness();
            }

            /// Once header is constructed, we can read size of data
            _function_sizeofdata = (sizeofdata_t) &VERSION_NAMESPACE::CURRENT_CLASS::sizeofdata;
        }

        /// If there is Trigger, set it
        if(_Header->getTriggerFilled()){
            void* pointer = &RAWBUFF32[_Header->sizeofdata()];

            EXECUTE_FUNCTION(setTriggerPointer)(pointer);

            /// If also, there is tail, get it
            if(_Header->getTailFilled()){
                pointer = &RAWBUFF32[sizeofdata() - _tailSize - 1];

                /// If tail exists, just assign the buffer source
                if(_Tail)
                    _Tail->setBufferSource(pointer);
                else
                    _Tail = new daqdataformats::RawEventTail(pointer, _Header->getTailVersion());
            }// end of filling tail
        }// end of filling trigger

        return true;
}



}}
