#include "DAQDataFormats/RawSummaryBlockHeader.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryBlockHeader
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE summaryblockheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

//------------------------------------------------------------------------------
RawSummaryBlockHeader::RawSummaryBlockHeader(const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Apply the default constructor
    defaultConstructorFixedSize();
}

//------------------------------------------------------------------------------
RawSummaryBlockHeader::RawSummaryBlockHeader(const RawSummaryBlockHeader& copy_in):
     PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawSummaryBlockHeader::RawSummaryBlockHeader(const void* summaryblockheader_buffer, const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    copy(summaryblockheader_buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawSummaryBlockHeader::~RawSummaryBlockHeader() { }

/// All binding routines
#define STANDARD_ROUTINES                                                   \
            BIND_GET_AND_SET(Version                  , version0, uint32_t);\
            BIND_GET_AND_SET(Marker                   , version0, uint32_t);\
            BIND_GET_AND_SET(TrigNumLo                , version0, uint32_t);\
            BIND_GET_AND_SET(TrigNumHi                , version0, uint32_t);\
            BIND_GET_AND_SET(TrigNum                  , version0, uint64_t);\
            BIND_GET_AND_SET(BuffId                   , version0, uint16_t);\
            BIND_GET_AND_SET(DataSize                 , version0, uint32_t);\
            BIND_GET_AND_SET(MonteCarloFlag           , version0, bool);    \
            BIND_IS_AND_SET(CRCCalculationUsed        , version0);          \
            BIND_IS_AND_SET(ConnectionInitialization  , version0);          \
            BIND_IS_AND_SET(CloseConnection           , version0);          \
            BIND_IS_AND_SET(MissingData               , version0);          \
            BIND_FUNCTION_TO_CUSTOM_FUNCTION(addDataSize          , CURRENT_CLASS, version0      , CURRENT_VERSION_NAMESPACE::CURRENT_CLASS::addDataSize          ,      , bool, const uint32_t);\
            BIND_FUNCTION_TO_CUSTOM_FUNCTION(checkMarker          , CURRENT_CLASS, version0      , CURRENT_VERSION_NAMESPACE::CURRENT_CLASS::checkMarker          , const, bool, ); \
            BIND_STADARD_FUNCTION(sizeofdata); \
            BIND_STADARD_FUNCTIONS;

#include "DAQDataFormats/setFunctionPointers_template.inc"

}// end of namespace
