#include <DatabaseUtils/DAQAppMgr/ApplicationType.h>

namespace dbutils {
namespace daqappmgr {

const std::string ApplicationType::BNEVB_APPTYPE("Buffer Node EVB");
const std::string ApplicationType::DL_APPTYPE("Data Logger");
const std::string ApplicationType::DCMAPP_APPTYPE("DCM Application");;
const std::string ApplicationType::GT_APPTYPE("Global Trigger");;

/**
 * Creates an instance from the specified values.
 */
ApplicationType::
ApplicationType(const std::string& typeName,
                const std::vector<std::string>& exeNameList,
                const std::string& startCommand,
                const std::string& rshCommandForStart,
                const std::string& stopCommand,
                const std::string& rshCommandForStop,
                const bool& supportsStatusRequest,
                const bool& hasAppNameInProcessString) :
    _typeName(typeName), _exeNameList(exeNameList),
    _startCommand(startCommand),
    _rshCommandForStart(rshCommandForStart),
    _stopCommand(stopCommand),
    _rshCommandForStop(rshCommandForStop),
    _supportsStatusRequest(supportsStatusRequest),
    _hasAppNameInProcessString(hasAppNameInProcessString)
{
}

/**
 * Tests if the specified instance is equal to this one.
 */
bool ApplicationType::
operator==(const ApplicationType& other) const
{
    if (other._supportsStatusRequest &&
        ! this->_supportsStatusRequest) {return false;}
    if (! other._supportsStatusRequest &&
        this->_supportsStatusRequest) {return false;}

    if (other._hasAppNameInProcessString &&
        ! this->_hasAppNameInProcessString) {return false;}
    if (! other._hasAppNameInProcessString &&
        this->_hasAppNameInProcessString) {return false;}

    if (other._exeNameList.size() != this->_exeNameList.size()) {return false;}
    for (int idx = 0; idx < (int) this->_exeNameList.size(); ++idx) {
        if (other._exeNameList[idx] != this->_exeNameList[idx]) {return false;}
    }

    return (other._typeName == this->_typeName &&
            other._startCommand == this->_startCommand &&
            other._rshCommandForStart == this->_rshCommandForStart &&
            other._stopCommand == this->_stopCommand &&
            other._rshCommandForStop == this->_rshCommandForStop);
}

/**
 * Tests if the specified set is not equal to this one.
 */
bool ApplicationType::
operator!=(const ApplicationType& other) const
{
    return !(*this==other);
}

/**
 * Tests if the specified instance is less than this one.
 */
bool ApplicationType::
operator<(const ApplicationType& other) const
{
    if (this->_typeName > other._typeName) {return false;}
    if (this->_typeName < other._typeName) {return true;}
    // else _typeName is equal

    if (this->_exeNameList.size() > other._exeNameList.size()) {return false;}
    if (this->_exeNameList.size() < other._exeNameList.size()) {return true;}
    for (int idx = 0; idx < (int) this->_exeNameList.size(); ++idx) {
        if (this->_exeNameList[idx] > other._exeNameList[idx]) {return false;}
        if (this->_exeNameList[idx] < other._exeNameList[idx]) {return true;}
    }
    // else _exeNameList contents are equal

    if (this->_startCommand > other._startCommand) {return false;}
    if (this->_startCommand < other._startCommand) {return true;}
    // else _startCommand is equal

    if (this->_rshCommandForStart > other._rshCommandForStart) {return false;}
    if (this->_rshCommandForStart < other._rshCommandForStart) {return true;}
    // else _rshCommandForStart is equal

    if (this->_stopCommand > other._stopCommand) {return false;}
    if (this->_stopCommand < other._stopCommand) {return true;}
    // else _stopCommand is equal

    if (this->_rshCommandForStop > other._rshCommandForStop) {return false;}
    if (this->_rshCommandForStop < other._rshCommandForStop) {return true;}
    // else _rshCommandForStop is equal

    if (! this->_supportsStatusRequest &&
        other._supportsStatusRequest) {return true;}
    if (this->_supportsStatusRequest &&
        ! other._supportsStatusRequest) {return false;}
    // else _supportsStatusRequest is equal

    if (! this->_hasAppNameInProcessString &&
        other._hasAppNameInProcessString) {return true;}
    else {return false;}
}

};  // end of namespace daqappmgr
};  // end of namespace dbutils
