/* 
 * File:   SHMConvention.cpp
 * Author: denis
 *
 * This class handles the memory segments conventions between DataLogger and Event Dispatcher
 *
 * Created on April 30, 2012, 4:10 PM
 */

#include <string>

#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>

#include "NovaDAQUtilities/SHMConvention.h"
#include "DAQDataFormats/BitFields.h"

using namespace novadaq;

//------------------------------------------------------------------------------
SHMConvention::SHMConvention() { }

//------------------------------------------------------------------------------
SHMConvention::~SHMConvention() { }

//------------------------------------------------------------------------------
// Getters
uint32_t SHMConvention::getNumberOfSHMSegments()  {return DEFAULT_SHM_SEGMENTS;}
uint32_t SHMConvention::getBaseSHMSize()          {return BASE_SHM_SIZE;}
uint32_t SHMConvention::getSHMSize()              {return SHM_memsize;}

//------------------------------------------------------------------------------
// Get shared memory memory segment ID based on partition and Offline framework ID
uint32_t SHMConvention::getSHMid(uint32_t partition, OFFLINE_FRAMEWORK framework)  {

    uint32_t id = 0;
	std::string sframework_hi=boost::lexical_cast<std::string>(framework/10);
	std::string sframework_low=boost::lexical_cast<std::string>(framework%10);
	std::string spartition=boost::lexical_cast<std::string>(partition);
    
    BITFIELDSET(id, p_letter           , MASK_1, SHIFT_1);
    BITFIELDSET(id, ((int) (*(spartition.data())))        , MASK_2, SHIFT_2);
    BITFIELDSET(id, ((int) (*(sframework_hi.data()))), MASK_3, SHIFT_3);
    BITFIELDSET(id, ((int) (*(sframework_low.data()))), MASK_4, SHIFT_4);

    return id;
}

