//  This file (getMilliBlock.cpp) was created by Ron Rechenmacher <ron@fnal.gov> on
//  Nov  4, 2011. "TERMS AND CONDITIONS" governing this file are in the README
//  or COPYING file. If you do not have such a file, one can be obtained by
//  contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
//  $RCSfile: ShmMilliBlock.cpp,v $
//  rev="$Revision: 1.7 $$Date: 2012/09/25 18:46:00 $";

#include <string.h>		// memcpy
#include "../include/ShmMilliBlock.h"
#include "DAQDataFormats/RawMilliBlockHeader.h"
#include "DAQDataFormats/RawMilliBlock.h"
using namespace daqdataformats;

bool
ShmMilliBlock::getMilliBlock(  uint32_t tmo_us
			     , RawMilliBlock & mb )
{
    uint64_t			start=shm_rd_.get_us_timeofday();
    shm_rdwr_buffer_header	*hdr;
    shm_rdwr_header             *shm_info_ptr;
    ShmRdWrCnt 			missed;
    uint32_t			cushion;
    int                 	new_tmo=tmo_us;
    std::vector<RawMilliSlice>	rms_vec;
    RawMilliSlice		rms_tmp(rawmillihdr::MILLISLICEHEADER_SIZE);

    mb.set_ovrwrt( false );

    shm_info_ptr = shm_rd_.get_ptr();
    if (shm_info_ptr == (void*)-1)
    { 
	if (shm_rd_.attach()!=0) return (false);
	shm_info_ptr = shm_rd_.get_ptr();
	my_area_.resize( shm_info_ptr->buf_sz_bytes/sizeof(uint32_t), 0 );
    }


    do
    {
	int sts=shm_rd_.start_read( &hdr, missed, cushion, new_tmo );
	if (sts > 0)
	{
            
	    memcpy( &my_area_[0], hdr, hdr->bytes_total );
	    sts = shm_rd_.check_overwrite( missed, cushion );
	    if (sts != 0)
	    {   // assume problem
	        mb.set_ovrwrt( true );
		break;
	    }
	    hdr = (shm_rdwr_buffer_header*)&my_area_[0];
	    RawMilliBlockHeader	*rmbhp=(RawMilliBlockHeader*)(hdr+1);
	    RawMilliBlockHeader	rmbh;
	    rmbh=*rmbhp;
	    void * msdataptr=(void*)(rmbhp+1);

	    rms_vec.reserve(rmbh.millislice_count);

	    uint32_t retrieved_millislice=0;
	    for (; retrieved_millislice<rmbh.millislice_count; ++retrieved_millislice)
	    {
		// get the millislice size
		rms_vec.push_back( rms_tmp );
		msdataptr = rms_vec[retrieved_millislice].readData( msdataptr );
	    }
	    //printf("retrieved_millislice=%u\n", retrieved_millislice );
	    shm_rd_.end_read( missed, cushion );
	    if (retrieved_millislice)
	    {   // If at least one slice, then rmbh stuff is good
		mb.set( rmbh.seqNum, rmbh.timeStart, rmbh.timeEnd
		       , rmbh.flags.truncated );
		mb.slices.swap( rms_vec );  // swap vectors
		return (true);
	    }
	    else
	    {   mb.set_tmo( false );
		return (false);
	    }
	}
    } while ((new_tmo=tmo_us-(shm_rd_.get_us_timeofday()-start)) > 0);
    mb.set_tmo( true );
    return (false);
}   // getMilliBlock
